/********************************************************************
NAME:     tchost   [ hostName | IPdotAddress ]

PURPOSE:
  This program is the rough equivalent of the AIX command "host"
  which can be used to help solve some network problems, mainly due
  to the domain name server.

NOTES:
 * To compile in HP-UX:        cc tchost.c -o tchost.hp
 * To compile in Solaris:      cc tchost.c -o tchost.sol
 * To compile in AIX:          cc tchost.c -o tchost.aix
 
FUTURE:
   It will be nice to add something equivalent to the "host" command 
   in AIX in which if the host name is an alias, 
   then it is identified as such.
*********************************************************************/

#include <stdio.h>
#include <netdb.h>
#include <netinet/in.h>
extern int h_errno;

/* Solaris does not define INADDR_NONE */
#ifndef INADDR_NONE
#define INADDR_NONE   0xffffffff  /* -1 return */
#endif

main(ac, av)
int ac;
char* av[];
{
  struct hostent  *hp;
  struct in_addr  addr;
  char		  *longAddr;
  char            *host;
  int             n = 0;
  unsigned long   ulongAddr;

  if (ac != 2)
  {
    printf("Usage: tchost [ hostName | IPdotAddress ]\n");
    printf("\n");
    printf("The tool tchost uses the gethostbyname and gethostbyaddr\n");
    printf("functions for the resolution of a host name or \n");
    printf("an IP dot address.\n");
    printf("This is useful to debug problems with the domain name server.\n");
    printf("It is recommended to use first the hostName and then to use\n");
    printf("the IP dot address obtained in the first invocation.\n");
    printf("\n");
    printf("Examples:\n");
    printf("       1) For a hostName:        tchost oem-ppc3\n");
    printf("       2) For an IP dot address: tchost 9.37.199.98\n");
    printf("       3) For the above invocations the output is:\n");
    printf("            oem-ppc3.raleigh.ibm.com is 9.37.199.98\n");
    return (1);
  }

  host = av[1];

  /* Obtain data for the host */
  hp = gethostbyname(host);

  /* For Windows only:
     Just in case that the user enters a dot address
         then use inet_addr() first, because gethostbyname
         in these environments do not recognize the dot address,
         only the hostName string.
  */
  ulongAddr = inet_addr(host);
  if ( ulongAddr == INADDR_NONE )
  {
    hp = gethostbyname(host);
    if (hp == NULL)
    {

      switch (h_errno) 
      {
        case HOST_NOT_FOUND:
          printf("The host is not found: %s\n",host);
          exit(1);
        case TRY_AGAIN:
          printf("The local server name is not active, try again later.\n");
          exit(1);
        case NO_RECOVERY:
          printf("Unrecoverable error obtained during gethostbyname\n");
          exit(1);
        case NO_ADDRESS:
          printf("There is no IP address for host %s\n",host);
          exit(1);
        default:
          printf("Unknown error during gethostbyname\n");
          exit(1);
      }
    }
  }
  else
  {
    hp = gethostbyaddr((char *)&ulongAddr,4, 2);  /* AF_INET = 2, TCP */

    if (hp == NULL)
    {
      switch (h_errno) 
      {
        case HOST_NOT_FOUND:
          printf("The IP address is not found: %s\n",host);
          exit(1);
        case TRY_AGAIN:
          printf("The local server name is not active, try again later.\n");
          exit(1);
        case NO_RECOVERY:
          printf("Unrecoverable error obtained during gethostbyaddr\n");
          exit(1);
        case NO_ADDRESS:
          printf("There is no host name for IP address %s\n",host);
          exit(1);
        default:
          printf("Unknown error during gethostbyaddr\n");
          exit(1);
      }
    }
  }


  /* Obtain data for the internet address */

  longAddr = hp->h_addr_list[n];

  /* Display the findings */

  addr = *(struct in_addr *) longAddr;
  (void) printf("%s is %s\n",hp->h_name,inet_ntoa(addr));

  /* Display aliases, if any */

  n++; /* get ready for the next alias, if any */
  while ( hp->h_addr_list[n] )
  {
     /* hp->h_addr_list[n] is the current address in host order. */
     addr = *(struct in_addr *) hp->h_addr_list[n];
     (void) printf("alias: %s\n",inet_ntoa(addr));
     n++;
  }

  return (0);
}

/* end of file */
